Class = require 'lib/Class'
Vector = require 'lib/Vector'

local START_ORBIT_MOD = 5/4
local MAX_ORBIT_MOD = 17/8
local ORBIT_CLAMP_MOD = 0.98

local Camera = Class({
	type = "Camera";
	
	focus = nil;

	orbit = {
		height = 0;
		height_vel = 0;
		angle = 0;
		angle_vel = 0;
	};

	pos = Vector(0.0, 0.0);
	angle = 0.0;
	zoom = 1.0;

	panning = false;
	last_mouse_x = 0;


	MousePressed = function(self, x, y, button)
		if button == 'r' then
			self.panning = true
			self.last_mouse_x = x
		end

		if button == 'wd' then
			self.orbit.height_vel = self.orbit.height_vel - 300
		elseif button == 'wu' then
			self.orbit.height_vel = self.orbit.height_vel + 300
		end
	end;

	MouseReleased = function(self, x, y, button)
		if button == 'r' then
			self.panning = false
		end
	end;

	SetFocus = function(self, new_focus)
		self.focus = new_focus
		if self.focus ~= nil then
			if self.focus.type == 'Planet' then
				print(self.focus.radius)
				self.orbit.height = self.focus.radius * START_ORBIT_MOD
			end
		end
	end;

	Update = function(self, dt)
		if self.focus ~= nil then
			if self.focus.type == 'Planet' then
				self.pos = self.focus:GetWorldPosition() + Vector(math.cos(self.orbit.angle) * self.orbit.height,
															 	  math.sin(self.orbit.angle) * self.orbit.height)
				self.angle = self.orbit.angle + math.pi/2

			elseif self.focus.type == 'StarSystem' then
				self.zoom = 0.05
			end
		end

		if self.panning then
			local diff = love.mouse.getX() - self.last_mouse_x
			self.last_mouse_x = love.mouse.getX()
			self.orbit.angle_vel = self.orbit.angle_vel - diff * 0.01 * dt * (self.orbit.height / math.abs(self.orbit.height))
		end
		self.orbit.angle = self.orbit.angle + self.orbit.angle_vel
		self.orbit.angle_vel = self.orbit.angle_vel * 0.94

		self.orbit.height = self.orbit.height + self.orbit.height_vel * dt
		self.orbit.height_vel = self.orbit.height_vel * 0.85

	end;

	Enable = function(self)
		local cx, cy = love.graphics.getWidth() / (2 * self.zoom), love.graphics.getHeight() / (2 * self.zoom)
		love.graphics.push()
		love.graphics.scale(self.zoom, self.zoom)
		love.graphics.translate(cx, cy)
		love.graphics.rotate(-self.angle)
		love.graphics.translate((-self.pos):unpack())
	end;

	Disable = function(self)
		love.graphics.pop()
	end;

	ToCameraCoords = function(self, vec)
		local w, h = love.graphics.getWidth(), love.graphics.getHeight()
		local c, s = math.cos(self.angle), math.sin(self.angle)
		local x, y = vec:unpack()
		--Do the translation
		x, y = x - self.pos.x, y - self.pos.y
		--Do the rotation
		x, y = c * x - s * y, s * x + c * y
		--Do the zoom and center offset in one step
		x, y = x * self.zoom + w / 2, y * self.zoom + h / 2
		return Vector(x, y)
	end;

	ToWorldCoords = function(self, vec)
		local w, h = love.graphics.getWidth(), love.graphics.getHeight()
		local c, s = math.cos(-self.angle), math.sin(-self.angle)
		local x, y = vec:unpack()
		--Undo the zoom
		x, y = (x - w/2) / self.zoom, (y - h/2) / self.zoom
		--Undo the rotation
		x, y = c*x - s*y, s*x + c*y

		return Vector(x + self.pos.x, y + self.pos.y)
	end;

	init = function(self)
		self.pos = Vector(0.0, 0.0)
	end
})

return Camera