local MAX_RECURSION_LEVELS = 5
local NIL = {}
local function Class(tab, ...)
	tab.___isclass = true
	local args = {...}
	local mt = {}
	mt.__call = (function(t, ...)
		t.__index = mt.__index
		if not t.__tostring then
			t.__tostring = function(s)
				if s.type ~= nil then
					return s.type
				end
				return "Class"
			end
		end
		--When you call "SomeClass()", this gets called, where ... are the arguments you passed
		--and t is definition of SomeClass (as returned by Class(tab, ...))
		--We're going to copy the definition of SomeClass to a new instance, piece by piece.
		recursive = function(a, b, key, level)
			if level > MAX_RECURSION_LEVELS then return end
			for k,v in pairs(b[key]) do
				a[key][k] = b[key][k]
				if type(v) == 'table' then
					a[key][k] = { }
					setmetatable(a[key][k], getmetatable(b[key][k]))
					recursive(a[key], b[key], k, level + 1)
				end
			end
		end
		local instance = {}
		for k,v in pairs(t) do
			instance[k] = t[k]
			if type(t[k]) == 'table' then
				instance[k] = { }
				setmetatable(instance[k], getmetatable(t[k]))
				recursive(instance, t, k, 1)
			end
		end
		--Now we run its init()...
		instance:init(...)
		return setmetatable(instance, t)
	end)
	mt.__index = (function(t, k)
		--called if index is not found in this table
		--so check base
		if #args == 1 then
			return args[1][k]
		elseif #args > 1 then
			for i,v in pairs(args) do
				if v[k] ~= nil then
					return v[k]
				end
				local mt = getmetatable(v)
				if mt[k] ~= nil then
					return mt[k]
				end
			end
		else
			return nil
		end
	end)

	--Make sure the class definition is what we're looking for
	if not tab.init then error("Tried to create class without init function") return nil end
	
	--Then return the definition as a table with our class metatable.
	return setmetatable(tab, mt)
end

--Then return this, so that the user may rename Class if they would like.
return Class